@@ -612,9 +612,9 @@ int scalar_int_op(eval_scalar *xr, const eval_scalar *xa, const eval_scalar *xb,
 		case EVAL_LT: r=to_qword(a<b); break;
 		case EVAL_LE: r=to_qword(a<=b); break;
 		// FIXME
-//		case EVAL_LAND: r=to_qword((a) && (b)); break;
-//		case EVAL_LXOR: r=to_qword((a && !b) || (!a && b)); break;
-//		case EVAL_LOR: r=to_qword(a||b); break;
+		case EVAL_LAND: r=to_qword(qword_cmp(a, to_qword(0)) && qword_cmp(b, to_qword(0))); break;
+		case EVAL_LXOR: r=to_qword((qword_cmp(a,to_qword(0)) && !qword_cmp(b,to_qword(0))) || (!qword_cmp(a,to_qword(0)) && qword_cmp(b,to_qword(0)))); break;
+		case EVAL_LOR: r=to_qword(qword_cmp(a,to_qword(0)) || qword_cmp(b,to_qword(0))); break;
 		default: 
 			set_eval_error("invalid operator");
 			return 0;
