@@ -27,24 +27,24 @@
 
 #define DESC_COFF			"coff - unix common obj file"
 #define DESC_COFF_HEADER		"coff/header"
-#define DESC_COFF_IMAGE		"coff/image"
+#define DESC_COFF_IMAGE			"coff/image"
 
-#define ATOM_COFF_MACHINES 				0xf0450000
+#define ATOM_COFF_MACHINES 			0xf0450000
 #define ATOM_COFF_MACHINES_STR			 "f0450000"
 
-#define ATOM_COFF_OPTIONAL_MAGICS 			0xf0450001
+#define ATOM_COFF_OPTIONAL_MAGICS 		0xf0450001
 #define ATOM_COFF_OPTIONAL_MAGICS_STR 		 "f0450001"
 
-#define ATOM_COFF_CHARACTERISTICS			0xf0450003
+#define ATOM_COFF_CHARACTERISTICS		0xf0450003
 #define ATOM_COFF_CHARACTERISTICS_STR		 "f0450003"
 
 #define ATOM_COFF_SECTION_CHARACTERISTICS	0xf0450004
-#define ATOM_COFF_SECTION_CHARACTERISTICS_STR "f0450004"
+#define ATOM_COFF_SECTION_CHARACTERISTICS_STR	 "f0450004"
 
 extern format_viewer_if htcoff_if;
 
 struct coff_section_headers {
-	UINT base_ofs;
+	UINT hdr_ofs;		// duplicate of ht_coff_shared_data.hdr_ofs (!)
 	UINT section_count;
 	COFF_SECTION_HEADER *sections;
 };
@@ -69,7 +69,7 @@ class ht_coff: public ht_format_group {
 private:
 	ht_coff_shared_data *coff_shared;
 public:
-			void init(bounds *b, ht_streamfile *file, format_viewer_if **ifs, ht_format_group *format_group, FILEOFS header_ofs);
+		void init(bounds *b, ht_streamfile *file, format_viewer_if **ifs, ht_format_group *format_group, FILEOFS header_ofs);
 	virtual	void done();
 };
 
@@ -82,4 +82,4 @@ int coff_ofs_to_rva(coff_section_headers *section_headers, dword ofs, RVA *rva);
 int coff_ofs_to_section(coff_section_headers *section_headers, dword ofs, UINT *section);
 int coff_ofs_to_rva_and_section(coff_section_headers *section_headers, dword ofs, RVA *rva, UINT *section);
 
-#endif /* !__HTPE_H__ */
+#endif /* !__HTCOFF_H__ */
