@@ -2,7 +2,7 @@
  *	HT Editor
  *	htendian.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -31,24 +31,10 @@
 
 enum endianess {big_endian, little_endian};
 void create_foreign_int(void *buf, int i, int size, endianess to_endianess);
+void create_foreign_int64(void *buf, const qword i, int size, endianess to_endianess);
 int create_host_int(const void *buf, int size, endianess from_endianess);
-void create_host_struct(void *buf, byte *table, endianess from);
-
-/* create network-order integer from host-order integer i */
-#define be2n_int h2n_int
-bool h2n_int(void *buf, int i, int size);
-
-/* create non-network-order integer from host-order integer i */
-#define le2n_int h2nn_int
-bool h2nn_int(void *buf, int i, int size);
-
-/* create host-order integer from network-order integer buf */
-#define be2h_int n2h_int
-bool n2h_int(int *i, const void *buf, int size);
-
-/* create host-order integer from non-network-order integer buf */
-#define le2h_int nn2h_int
-bool nn2h_int(int *i, const void *buf, int size);
+qword create_host_int64(const void *buf, endianess from_endianess);
+void create_host_struct(void *buf, const byte *table, endianess from);
 
 #endif /* __HTENDIAN_H__ */
 
