@@ -2,7 +2,7 @@
  *	HT Editor
  *	htexcept.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -38,15 +38,21 @@ public:
  *	CLASS ht_io_exception
  */
 
-#define HT_IO_EXCEPTION_MAX_ERRSTR		256
+#define HT_MSG_EXCEPTION_MAX_ERRSTR		256
 
-class ht_io_exception: public ht_exception {
+class ht_msg_exception: public ht_exception {
 protected:
-	char estr[HT_IO_EXCEPTION_MAX_ERRSTR];
+	char estr[HT_MSG_EXCEPTION_MAX_ERRSTR];
 public:
-			    ht_io_exception(char *errstr,...);
+			    ht_msg_exception(char *errstr,...);
 /* new */
 	virtual const char *what();
 };
 
+/*
+ *	CLASS ht_io_exception
+ */
+
+#define ht_io_exception ht_msg_exception
+
 #endif /* __HTEXCEPT_H__ */
