@@ -2,7 +2,7 @@
  *	HT Editor
  *	hthex.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,7 @@
 
 #include "formats.h"
 
-#define HEX_DESC "hex"
+#define DESC_HEX "hex"
 
 extern format_viewer_if hthex_if;
 
@@ -31,14 +31,18 @@ extern format_viewer_if hthex_if;
  *	CLASS ht_hex_viewer
  */
 
+class ht_hex_file_sub;
 class ht_hex_viewer: public ht_uformat_viewer {
+public:
+	ht_hex_file_sub *h;
 protected:
-	virtual	bool address_to_offset(fmt_vaddress addr, FILEOFS *ofs);
-	virtual	char *func(UINT i, bool execute);
 	virtual	void get_pindicator_str(char *buf);
-	virtual	bool get_hscrollbar_pos(int *pstart, int *psize);
+	virtual	bool get_vscrollbar_pos(int *pstart, int *psize);
 	virtual	void handlemsg(htmsg *msg);
-	virtual	bool offset_to_address(FILEOFS ofs, fmt_vaddress *addr);
+	virtual	bool offset_to_pos(FILEOFS ofs, viewer_pos *p);
+	virtual	bool pos_to_offset(viewer_pos p, FILEOFS *ofs);
+	virtual	bool qword_to_pos(qword q, viewer_pos *pos);
+	virtual	int symbol_handler(eval_scalar *result, char *name);
 };
 
 /*
