@@ -2,7 +2,7 @@
  *	HT Editor
  *	htmz.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -25,7 +25,7 @@
 
 #include "mzstruct.h"
 
-#define DESC_MZ "mz"
+#define DESC_MZ "mz - dos exe"
 #define DESC_MZ_HEADER "mz/header"
 #define DESC_MZ_REL "mz/relocations"
 #define DESC_MZ_IMAGE "mz/image"
