@@ -2,7 +2,7 @@
  *	HT Editor
  *	htobj.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -19,14 +19,17 @@
  */
 
 #include "htatom.h"
+#include "cmds.h"
 #include "htapp.h"
 #include "htctrl.h"
 #include "htdebug.h"
 #include "htkeyb.h"
+#include "htmenu.h"
 #include "htobj.h"
 #include "htpal.h"
 #include "htreg.h"
 #include "htstring.h"
+#include "snprintf.h"
 #include "store.h"
 #include "tools.h"
 
@@ -41,6 +44,9 @@
 #define ATOM_HT_FRAME		MAGICD("OBJ\4")
 #define ATOM_HT_SCROLLBAR	MAGICD("OBJ\5")
 
+#define DEFAULT_VIEW_MIN_WIDTH 	25
+#define DEFAULT_VIEW_MIN_HEIGHT	6
+
 void bounds_and(bounds *a, bounds *b)
 {
 	if (b->x>a->x) {
@@ -59,10 +65,10 @@ void bounds_and(bounds *a, bounds *b)
 
 void put_bounds(ht_object_stream *s, bounds *b)
 {
-	s->put_int_dec(b->x, 4, NULL);
-	s->put_int_dec(b->y, 4, NULL);
-	s->put_int_dec(b->w, 4, NULL);
-	s->put_int_dec(b->h, 4, NULL);
+	s->putIntDec(b->x, 4, NULL);
+	s->putIntDec(b->y, 4, NULL);
+	s->putIntDec(b->w, 4, NULL);
+	s->putIntDec(b->h, 4, NULL);
 }
 
 void clearmsg(htmsg *msg)
@@ -75,7 +81,7 @@ void clearmsg(htmsg *msg)
  *	CLASS ht_text
  */
 
-void ht_text::settext(char *text)
+void ht_text::settext(const char *text)
 {
 }
 
@@ -83,45 +89,45 @@ void ht_text::settext(char *text)
  *	CLASS ht_view
  */
 
-void ht_view::init(bounds *b, int _options, char *_desc)
+void ht_view::init(bounds *b, int o, const char *d)
 {
-	object::init();
+	Object::init();
 	VIEW_DEBUG_NAME("ht_view");
-	desc=ht_strdup(_desc);
-	group=0;
-	focused=0;
-	browse_idx=0;
-	view_is_dirty=1;
-	size.x=0;
-	size.y=0;
-	size.w=0;
-	size.h=0;
-	prev=0;
-	next=0;
-	setoptions(_options);
-	buf=0;
-	enabled=1;
+	desc = ht_strdup(d);
+	group = 0;
+	focused = 0;
+	browse_idx = 0;
+	view_is_dirty = true;
+	size.x = 0;
+	size.y = 0;
+	size.w = 0;
+	size.h = 0;
+	prev = NULL;
+	next = NULL;
+	setoptions(o);
+	buf = 0;
+	enabled = true;
 
-	growmode = GM_TOP |  GM_LEFT;
+	growmode = MK_GM(GMH_LEFT, GMV_TOP);
 	
 	if (options & VO_OWNBUFFER) {
-		buf=new drawbuf(&size);
+		buf = new drawbuf(&size);
 		enable_buffering();
 	} else {
-		buf=screen;
+		buf = screen;
 		disable_buffering();
 	}
 
-	g_hdist=0;
-	g_vdist=0;
+	g_hdist = 0;
+	g_vdist = 0;
 
 	setbounds(b);
 
-	pal.data=0;
-	pal.size=0;
+	pal.data = NULL;
+	pal.size = 0;
 
-	pal_class=defaultpaletteclass();
-	pal_name=defaultpalette();
+	pal_class = defaultpaletteclass();
+	pal_name = defaultpalette();
 
 	reloadpalette();
 }
@@ -131,7 +137,7 @@ void ht_view::done()
 	if (desc) free(desc);
 	if (pal.data) free(pal.data);
 	if (options & VO_OWNBUFFER) delete buf;
-	object::done();
+	Object::done();
 }
 
 int ht_view::alone()
@@ -200,10 +206,10 @@ void ht_view::buf_printchar(int x, int y, int c, int ch)
 
 int ht_view::buf_printf(int x, int y, int c, char *format, ...)
 {
-	char buf[1024];	/* FIXME: possible buffer overflow ! */
+	char buf[256];	/* secure */
 	va_list arg;
 	va_start(arg, format);
-	vsprintf(buf, format, arg);
+	ht_vsnprintf(buf, sizeof buf, format, arg);
 	va_end(arg);
 	return buf_print(x, y, c, buf);
 }
@@ -267,12 +273,12 @@ int ht_view::datasize()
 
 char *ht_view::defaultpalette()
 {
-	return palkey_windialog_window_default;
+	return palkey_generic_window_default;
 }
 
 char *ht_view::defaultpaletteclass()
 {
-	return palclasskey_windialog;
+	return palclasskey_generic;
 }
 
 void ht_view::dirtyview()
@@ -404,6 +410,12 @@ vcp ht_view::getcolor(UINT index)
 	return getcolorv(&pal, index);
 }
 
+void ht_view::getminbounds(int *width, int *height)
+{
+	*width = DEFAULT_VIEW_MIN_WIDTH;
+	*height = DEFAULT_VIEW_MIN_HEIGHT;
+}
+
 struct databufdup_s {
 	ht_memmap_file *f;
 	ht_object_stream_memmap *s;
@@ -422,10 +434,10 @@ void ht_view::databuf_freedup(void *handle)
 	free(s);
 }
 
-void ht_view::databuf_get(void *buf)
+void ht_view::databuf_get(void *buf, int bufsize)
 {
 	ht_memmap_file *f=new ht_memmap_file();
-	f->init((byte*)buf);
+	f->init((byte*)buf, bufsize);
 	
 	ht_object_stream_memmap *s=new ht_object_stream_memmap();
 	s->init(f, false);
@@ -439,10 +451,10 @@ void ht_view::databuf_get(void *buf)
 	delete f;
 }
 
-void *ht_view::databuf_getdup(void *buf)
+void *ht_view::databuf_getdup(void *buf, int bufsize)
 {
 	ht_memmap_file *f=new ht_memmap_file();
-	f->init((byte*)buf);
+	f->init((byte*)buf, bufsize);
 	
 	ht_object_stream_memmap *s=new ht_object_stream_memmap();
 	s->init(f, true);
@@ -455,10 +467,10 @@ void *ht_view::databuf_getdup(void *buf)
 	return q;
 }
 
-void ht_view::databuf_set(void *buf)
+void ht_view::databuf_set(void *buf, int bufsize)
 {
 	ht_memmap_file *f=new ht_memmap_file();
-	f->init((byte*)buf);
+	f->init((byte*)buf, bufsize);
 	
 	ht_object_stream_memmap *s=new ht_object_stream_memmap();
 	s->init(f, false);
@@ -553,7 +565,7 @@ void ht_view::move(int rx, int ry)
 	app->clipbounds(&vsize);
 }
 
-OBJECT_ID ht_view::object_id()
+OBJECT_ID ht_view::object_id() const
 {
 	return ATOM_HT_VIEW;
 }
@@ -585,40 +597,20 @@ void ht_view::redraw()
 	}
 }
 
-void ht_view::resize(int rw, int rh)
+void ht_view::resize(int sx, int sy)
 {
 	if (options & VO_RESIZE) {
-		if (size.w+rw-1<0) rw=-size.w+1;
-		if (size.h+rh-1<0) rh=-size.h+1;
-		size.w+=rw;
-		size.h+=rh;
-		buf->b_resize(rw, rh);
+		if (size.w+sx <= 0) sx=-size.w+1;
+		if (size.h+sy <= 0) sy=-size.h+1;
+		size.w+=sx;
+		size.h+=sy;
+		buf->b_resize(sx, sy);
 	}
-	vsize=size;
+	vsize = size;
 	if (group) group->clipbounds(&vsize);
 	app->clipbounds(&vsize);
 }
 
-void ht_view::resize_group(int rx, int ry)
-{
-	int px=0, py=0;
-	int sx=0, sy=0;
-	if (growmode & GM_HDEFORM) {
-		sx=rx;
-	} else if (growmode & GM_RIGHT) {
-		px=rx;
-	} else /* GM_LEFT */ {
-	}
-	if (growmode & GM_VDEFORM) {
-		sy=ry;
-	} else if (growmode & GM_BOTTOM) {
-		py=ry;
-	} else /* GM_TOP */ {
-	}
-	move(px, py);
-	resize(sx, sy);
-}
-
 void ht_view::releasefocus()
 {
 	dirtyview();
@@ -632,7 +624,7 @@ void ht_view::reloadpalette()
 		free(pal.data);
 		pal.data=0;
 	}	    
-	reloadpalettev(pal_class, pal_name, &pal);
+	load_pal(pal_class, pal_name, &pal);
 }
 
 void ht_view::relocate_to(ht_view *view)
@@ -670,7 +662,7 @@ void ht_view::sendmsg(int msg, void *data1, void *data2)
 	sendmsg(&m);
 }
 
-void ht_view::sendmsg(int msg, int data1=0, int data2=0)
+void ht_view::sendmsg(int msg, int data1, int data2)
 {
 	htmsg m;
 	switch (msg) {
@@ -707,10 +699,21 @@ void ht_view::setvisualbounds(bounds *b)
 	}
 }
 
-void ht_view::setcursor(int x, int y)
+void ht_view::setcursor(int x, int y, cursor_mode c)
 {
-	if (pointvisible(size.x+x, size.y+y)) screen->setcursor(size.x+x, size.y+y); else
+	if (pointvisible(size.x+x, size.y+y)) {
+		screen->setcursor(size.x+x, size.y+y);
+		switch (c) {
+			case cm_normal:
+				screen->setcursormode(0);
+				break;
+			case cm_overwrite:
+				screen->setcursormode(1);
+				break;
+		}
+	} else {
 		screen->hidecursor();
+	}
 }
 
 void ht_view::setdata(ht_object_stream *s)
@@ -726,14 +729,14 @@ void ht_view::setnumber(UINT number)
 {
 }
 
-void ht_view::setoptions(int _options)
+void ht_view::setoptions(int Options)
 {
-	options=_options;
+	options = Options;
 }
 
-void ht_view::setpalette(char *_pal_name)
+void ht_view::setpalette(char *Pal_name)
 {
-	pal_name=_pal_name;
+	pal_name = Pal_name;
 	reloadpalette();
 }
 
@@ -745,16 +748,16 @@ void ht_view::setpalettefull(char *_pal_name, char *_pal_class)
 
 void	ht_view::store(ht_object_stream *s)
 {
-	s->put_bool(enabled, NULL);
-	s->put_bool(focused, NULL);
-	s->put_int_dec(options, 4, NULL);
-	s->put_int_dec(browse_idx, 4, NULL);
-	s->put_string(desc, NULL);
+/*	s->putBool(enabled, NULL);
+	s->putBool(focused, NULL);
+	s->putIntDec(options, 4, NULL);
+	s->putIntDec(browse_idx, 4, NULL);
+	s->putString(desc, NULL);
 	put_bounds(s, &size);
 	put_bounds(s, &vsize);
-	s->put_string(pal_class, NULL);
-	s->put_string(pal_name, NULL);
-	s->put_int_dec(growmode, 4, NULL);
+	s->putString(pal_class, NULL);
+	s->putString(pal_name, NULL);
+	s->putIntDec(growmode, 4, NULL);*/
 }
 
 void ht_view::unrelocate_to(ht_view *view)
@@ -770,7 +773,7 @@ void ht_view::unrelocate_to(ht_view *view)
  *	CLASS ht_group
  */
 
-void ht_group::init(bounds *b, int options, char *desc)
+void ht_group::init(bounds *b, int options, const char *desc)
 {
 	first=0;
 	current=0;
@@ -780,13 +783,7 @@ void ht_group::init(bounds *b, int options, char *desc)
 	view_count=0;
 	shared_data=0;
 
-	growmode = GM_HDEFORM | GM_VDEFORM;
-
-/* FIXME: debug */
-	if (options & VO_OWNBUFFER) {
-		HT_WARN("class 'ht_group' should not have own buffer !");
-	}
-/**/
+	growmode = MK_GM(GMH_FIT, GMV_FIT);
 }
 
 void ht_group::done()
@@ -877,7 +874,7 @@ int ht_group::focusnext()
 		if (i>view_count-1) i=0;
 		if (i==current->browse_idx) break;
 		ht_view *v=get_by_browse_idx(i);
-		if (v->options & VO_SELECTABLE) {
+		if (v && (v->options & VO_SELECTABLE)) {
 			x=v;
 			break;
 		}
@@ -905,7 +902,7 @@ int ht_group::focusprev()
 		if (i<0) i=view_count-1;
 		if (i==current->browse_idx) break;
 		ht_view *v=get_by_browse_idx(i);
-		if (v->options & VO_SELECTABLE) {
+		if (v && (v->options & VO_SELECTABLE)) {
 			v->selectlast();
 			focus(v);
 			return 1;
@@ -1094,7 +1091,7 @@ void ht_group::move(int rx, int ry)
 	}
 }
 
-OBJECT_ID ht_group::object_id()
+OBJECT_ID ht_group::object_id() const
 {
 	return ATOM_HT_GROUP;
 }
@@ -1142,14 +1139,49 @@ void ht_group::remove(ht_view *view)
 	if (last==view) last=last->prev;
 }
 
+void ht_group::reorder_view(ht_view *v, int rx, int ry)
+{
+	int px=0, py=0;
+	int sx=0, sy=0;
+
+	int gmv = GET_GM_V(v->growmode);
+	int gmh = GET_GM_H(v->growmode);
+	switch (gmh) {
+		case GMH_LEFT:
+			/* do nothing */
+			break;
+		case GMH_RIGHT:
+			px = rx;
+			break;
+		case GMH_FIT:
+			sx = rx;
+			break;
+	}
+	
+	switch (gmv) {
+		case GMV_TOP:
+			/* do nothing */
+			break;
+		case GMV_BOTTOM:
+			py = ry;
+			break;
+		case GMV_FIT:
+			sy = ry;
+			break;
+	}
+
+	v->move(px, py);
+	v->resize(sx, sy);
+}
+
 void ht_group::resize(int rx, int ry)
 {
 	ht_view::resize(rx, ry);
 	
-	ht_view *v=first;
+	ht_view *v = first;
 	while (v) {
-		v->resize_group(rx, ry);
-		v=v->next;
+		reorder_view(v, rx, ry);
+		v = v->next;
 	}
 }
 
@@ -1217,10 +1249,10 @@ void ht_group::setpalette(char *pal_name)
 void ht_group::store(ht_object_stream *s)
 {
 	ht_view::store(s);
-	s->put_int_dec(childcount(), 4, NULL);
+	s->putIntDec(childcount(), 4, NULL);
 	ht_view *v=first;
 	while (v) {
-		s->put_object(v, NULL);
+		s->putObject(v, NULL);
 		v=v->next;
 	}
 }
@@ -1229,7 +1261,7 @@ void ht_group::store(ht_object_stream *s)
  *	CLASS ht_xgroup
  */
 
-void ht_xgroup::init(bounds *b, int options, char *desc)
+void ht_xgroup::init(bounds *b, int options, const char *desc)
 {
 	ht_group::init(b, options, desc);
 	VIEW_DEBUG_NAME("ht_xgroup");
@@ -1250,7 +1282,7 @@ int ht_xgroup::countselectables()
 
 void ht_xgroup::handlemsg(htmsg *msg)
 {
-	if (msg->type==mt_broadcast) {
+	if ((msg->msg!=msg_draw) && (msg->type==mt_broadcast)) {
 		ht_group::handlemsg(msg);
 	} else {
 		if (msg->msg==msg_complete_init) return;
@@ -1270,7 +1302,7 @@ int ht_xgroup::load(ht_object_stream *s)
 	return ht_group::load(s);
 }
 
-OBJECT_ID ht_xgroup::object_id()
+OBJECT_ID ht_xgroup::object_id() const
 {
 	return ATOM_HT_XGROUP;
 }
@@ -1304,12 +1336,13 @@ void	ht_xgroup::store(ht_object_stream *s)
 bool scrollbar_pos(int start, int size, int all, int *pstart, int *psize)
 {
 	if (!all) return false;
-	*psize=size*100/all;
 	if (start+size>=all) {
 		if (size>=all) return false;
-		*pstart=100-*psize;
+		*psize = (int)(((double)size)*100/all);
+		*pstart = 100-*psize;
 	} else {
-		*pstart=start*100/all;
+		*psize = (int)(((double)size)*100/all);
+		*pstart = (int)(((double)start)*100/all);
 	}
 	return true;
 }
@@ -1327,9 +1360,9 @@ void	ht_scrollbar::init(bounds *b, palette *p, bool isv)
 	isvertical=isv;
 
 	if (isvertical) {
-		growmode=GM_BOTTOM | GM_HDEFORM;
+		growmode = MK_GM(GMH_RIGHT, GMV_FIT);
 	} else {
-		growmode=GM_RIGHT | GM_VDEFORM;
+		growmode = MK_GM(GMH_FIT, GMV_BOTTOM);
 	}
 
 	enable();	// enabled by default
@@ -1357,12 +1390,10 @@ void ht_scrollbar::disable()
 void ht_scrollbar::draw()
 {
 	if (enabled) {
-		vcp color=getcolorv(gpal, palidx_windialog_scrollbar);
+		vcp color = getcolorv(gpal, palidx_generic_scrollbar);
+//          vcp color2 = VCP(VCP_BACKGROUND(color), VCP_FOREGROUND(color));
 		if (isvertical) {
-		} else {
-			buf_printchar(0, 0, color, CHAR_ARROWBIG_UP);
 			fill(0, 1, size.w, size.h-2, color, ' ');
-			buf_printchar(0, size.h-1, color, CHAR_ARROWBIG_DOWN);
 			int e, s;
 			e=((size.h-2)*psize)/100;
 			if (pstart+psize>=100) {
@@ -1375,6 +1406,9 @@ void ht_scrollbar::draw()
 				e=1;
 			}
 			fill(0, s+1, 1, e, color, CHAR_FILLED_M);
+			buf_printchar(0, 0, color, CHAR_ARROWBIG_UP);
+			buf_printchar(0, size.h-1, color, CHAR_ARROWBIG_DOWN);
+		} else {
 		}
 	}
 }
@@ -1384,7 +1418,7 @@ int ht_scrollbar::load(ht_object_stream *s)
 	return 1;
 }
 
-OBJECT_ID ht_scrollbar::object_id()
+OBJECT_ID ht_scrollbar::object_id() const
 {
 	return ATOM_HT_SCROLLBAR;
 }
@@ -1404,16 +1438,16 @@ void	ht_scrollbar::store(ht_object_stream *s)
  *	CLASS ht_frame
  */
 
-void ht_frame::init(bounds *b, char *desc, UINT _style, UINT _number=0)
+void ht_frame::init(bounds *b, const char *desc, UINT s, UINT n)
 {
 	ht_view::init(b, VO_RESIZE, desc);
 	VIEW_DEBUG_NAME("ht_frame");
 
-	number=_number;
-	style=_style;
-	framestate=FST_UNFOCUSED;
+	number = n;
+	style = s;
+	framestate = FST_UNFOCUSED;
 	
-	growmode=GM_HDEFORM | GM_VDEFORM;
+	growmode = MK_GM(GMH_FIT, GMV_FIT);
 }
 
 void ht_frame::done()
@@ -1454,11 +1488,11 @@ void ht_frame::draw()
 /* "..." */
 	buf_printchar(0, size.h-1, c, cornerll);
 	for (int i=1; i<size.w-1; i++) buf_printchar(i, size.h-1, c, lineh);
-/*	if (style & FS_MOVE) {
-		buf_printchar(size.w-1, size.h-1, getcurcol_killer(), cornerlr);
+/*	if (style & FS_RESIZE) {
+		buf_printchar(size.w-1, size.h-1, getcurcol_killer(), CHAR_CORNERLR);
 	} else {*/
 		buf_printchar(size.w-1, size.h-1, c, cornerlr);
-/*     }*/
+//     }
 /* "", "" */
 	for (int i=1; i<size.h-1; i++) {
 		buf_printchar(0, i, c, linev);
@@ -1484,10 +1518,10 @@ void ht_frame::draw()
 	char *d;
 	switch (framestate) {
 		case FST_MOVE:
-			d = "cursor keys move";
+			d = (char*)((style & FS_RESIZE) ? "(moving) - hit space to resize" : "(moving)");
 			break;
 		case FST_RESIZE:
-			d = "cursor keys resize";
+			d = (char*)((style & FS_MOVE) ? "(resizing) - hit space to move" : "(resizing)");
 			break;
 		default:
 			d = desc;
@@ -1496,7 +1530,7 @@ void ht_frame::draw()
 	ns++;
 	if ((d) && (style & FS_TITLE)) {
 		int l = strlen(d), k = 0;
-		if (l > size.w-(6+ks+ns)) {
+		if (l > size.w-(5+ks+ns)) {
 			k = l-(size.w-(6+ks+ns+2));
 			if (size.w > 6+ks+ns+2) {
 				d+=k;
@@ -1512,19 +1546,19 @@ vcp ht_frame::getcurcol_normal()
 {
 	switch (framestate) {
 		case FST_FOCUSED:
-			return getcolor(palidx_windialog_frame_focused);
+			return getcolor(palidx_generic_frame_focused);
 		case FST_UNFOCUSED:
-			return getcolor(palidx_windialog_frame_unfocused);
+			return getcolor(palidx_generic_frame_unfocused);
 		case FST_MOVE:
 		case FST_RESIZE:
-			return getcolor(palidx_windialog_frame_moved);
+			return getcolor(palidx_generic_frame_move_resize);
 	}
 	return 0;
 }
 
 vcp ht_frame::getcurcol_killer()
 {
-	return getcolor(palidx_windialog_frame_killer);
+	return getcolor(palidx_generic_frame_killer);
 }
 
 UINT ht_frame::getnumber()
@@ -1542,7 +1576,7 @@ int ht_frame::load(ht_object_stream *s)
 	return ht_view::load(s);
 }
 
-OBJECT_ID ht_frame::object_id()
+OBJECT_ID ht_frame::object_id() const
 {
 	return ATOM_HT_FRAME;
 }
@@ -1564,10 +1598,11 @@ void ht_frame::setstyle(UINT s)
 	style=s;
 }
 
-void ht_frame::settitle(char *title)
+void ht_frame::settext(const char *text)
 {
 	if (desc) free(desc);
-	desc=ht_strdup(title);
+	desc = ht_strdup(text);
+	dirtyview();
 }
 
 void ht_frame::store(ht_object_stream *s)
@@ -1579,7 +1614,7 @@ void ht_frame::store(ht_object_stream *s)
  *	CLASS ht_window
  */
 
-void	ht_window::init(bounds *b, char *desc, UINT framestyle, UINT num)
+void	ht_window::init(bounds *b, const char *desc, UINT framestyle, UINT num)
 {
 	ht_group::init(b, VO_SELECTABLE | VO_SELBOUND | VO_BROWSABLE, desc);
 	VIEW_DEBUG_NAME("ht_window");
@@ -1607,7 +1642,7 @@ void ht_window::done()
 
 void ht_window::draw()
 {
-	vcp c=getcolor(palidx_windialog_body);
+	vcp c=getcolor(palidx_generic_body);
 	clear(c);
 }
 
@@ -1627,6 +1662,11 @@ UINT ht_window::getnumber()
 	return number;
 }
 
+ht_frame *ht_window::getframe()
+{
+	return frame;
+}
+
 void ht_window::handlemsg(htmsg *msg)
 {
 	switch (msg->msg) {
@@ -1666,15 +1706,17 @@ void ht_window::handlemsg(htmsg *msg)
 				}
 			} else if (action_state==WAC_RESIZE) {
 				if (options & VO_RESIZE) {
+					int min_width, min_height;
+					getminbounds(&min_width, &min_height);
 					switch (msg->data1.integer) {
 						case K_Up:
-							if (size.h>/*MIN(*/7/*, isize.h)*/) resize(0, -1);
+							if (size.h > min_height) resize(0, -1);
 							break;
 						case K_Down:
-							if (size.h<group->size.h) resize(0, 1);
+							if (size.h < group->size.h) resize(0, 1);
 							break;
 						case K_Left:
-							if ((size.x+size.w>1) && (size.w>/*MIN(*/25/*, isize.w)*/)) resize(-1, 0);
+							if ((size.x+size.w>1) && (size.w > min_width)) resize(-1, 0);
 							break;
 						case K_Right:
 							if (size.w<group->size.w) resize(1, 0);
@@ -1682,7 +1724,8 @@ void ht_window::handlemsg(htmsg *msg)
 					}
 				}
 			} else {
-				if (msg->data1.integer == K_Control_F5) {
+				if ((msg->data1.integer == K_Control_F5) ||
+				 (msg->data1.integer == K_Control_R)) {
 					sendmsg(cmd_window_resizemove);
 				}
 				break;
@@ -1759,13 +1802,32 @@ bool ht_window::next_action_state()
 	return false;
 }
 
-OBJECT_ID ht_window::object_id()
+OBJECT_ID ht_window::object_id() const
 {
 	return ATOM_HT_WINDOW;
 }
 
 void ht_window::receivefocus()
 {
+	htmsg m;
+	m.msg = msg_contextmenuquery;
+	m.type = mt_empty;
+	sendmsg(&m);
+	ht_menu *q = (ht_menu*)((ht_app*)app)->menu;
+	if (m.msg == msg_retval) {
+		ht_context_menu *n = (ht_context_menu*)m.data1.ptr;
+		if (q) {
+			if (!q->set_local_menu(n)) {
+				n->done();
+				delete n;
+			}
+			q->sendmsg(msg_dirtyview);
+		} else {
+			n->done();
+			delete n;
+		}
+	}
+
 	ht_group::receivefocus();
 	if (frame) frame->setstyle(frame->getstyle() | FS_THICK);
 }
@@ -1777,26 +1839,26 @@ void ht_window::redraw()
 	char buf[256];
 	buf[0]=0;
 	
-	m.msg=msg_get_scrollinfo;
-	m.type=mt_empty;
-	m.data1.integer=gsi_pindicator;
-	m.data2.ptr=buf;
+	m.msg = msg_get_scrollinfo;
+	m.type = mt_empty;
+	m.data1.integer = gsi_pindicator;
+	m.data2.ptr = buf;
 	sendmsg(&m);
 
 	if (pindicator) pindicator->settext(buf);
 
 	gsi_scrollbar_t p;
 
-	p.pstart=0;
-	p.psize=200;
-	m.msg=msg_get_scrollinfo;
-	m.type=mt_empty;
-	m.data1.integer=gsi_hscrollbar;
-	m.data2.ptr=&p;
+	p.pstart = 0;
+	p.psize = 200;
+	m.msg = msg_get_scrollinfo;
+	m.type = mt_empty;
+	m.data1.integer = gsi_hscrollbar;
+	m.data2.ptr = &p;
 	sendmsg(&m);
 
 	if (hscrollbar) {
-		if (p.psize>100) {
+		if (p.psize>=100) {
 			hscrollbar->disable();
 		} else {
 			hscrollbar->enable();
@@ -1804,16 +1866,16 @@ void ht_window::redraw()
 		}
 	}
 
-	p.pstart=0;
-	p.psize=200;
-	m.msg=msg_get_scrollinfo;
-	m.type=mt_empty;
-	m.data1.integer=gsi_vscrollbar;
-	m.data2.ptr=&p;
+	p.pstart = 0;
+	p.psize = 200;
+	m.msg = msg_get_scrollinfo;
+	m.type = mt_empty;
+	m.data1.integer = gsi_vscrollbar;
+	m.data2.ptr = &p;
 	sendmsg(&m);
 
 	if (vscrollbar) {
-		if (p.psize>100) {
+		if (p.psize>=100) {
 			vscrollbar->disable();
 		} else {
 			vscrollbar->enable();
@@ -1826,6 +1888,12 @@ void ht_window::redraw()
 
 void ht_window::releasefocus()
 {
+	ht_menu *q = (ht_menu*)((ht_app*)app)->menu;
+	if (q) {
+		q->delete_local_menu();
+		q->sendmsg(msg_dirtyview);
+	}
+
 	if (frame) frame->setstyle(frame->getstyle() & (~FS_THICK));
 	ht_group::releasefocus();
 }
@@ -1882,7 +1950,7 @@ void ht_window::settitle(char *title)
 {
 	if (desc) free(desc);
 	desc=ht_strdup(title);
-	if (frame) frame->settitle(title);
+	if (frame) frame->settext(title);
 }
 
 void ht_window::setvscrollbar(ht_scrollbar *s)
@@ -1904,7 +1972,7 @@ void	ht_window::store(ht_object_stream *s)
 
 void ht_vbar::draw()
 {
-	fill(0, 0, 1, size.h, getcolor(palidx_windialog_body), CHAR_LINEV);
+	fill(0, 0, 1, size.h, getcolor(palidx_generic_body), CHAR_LINEV);
 }
 
 /*
@@ -1913,7 +1981,7 @@ void ht_vbar::draw()
 
 void ht_hbar::draw()
 {
-	fill(0, 0, size.w, 1, getcolor(palidx_windialog_body), CHAR_LINEH);
+	fill(0, 0, size.w, 1, getcolor(palidx_generic_body), CHAR_LINEH);
 }
 
 /***/
