@@ -2,7 +2,7 @@
  *	HT Editor
  *	htpeimg.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -21,8 +21,22 @@
 #ifndef __HTPEIMG_H__
 #define __HTPEIMG_H__
 
+#include "htanaly.h"
+#include "htpe.h"
 #include "formats.h"
 
 extern format_viewer_if htpeimage_if;
 
+/*
+ *	CLASS ht_pe_aviewer
+ */
+
+class ht_pe_aviewer: public ht_aviewer {
+public:
+	ht_pe_shared_data *pe_shared;
+		   void init(bounds *b, char *desc, int caps, ht_streamfile *file, ht_format_group *format_group, Analyser *Analyser, ht_pe_shared_data *pe_shared);
+	virtual int  func_handler(eval_scalar *result, char *name, eval_scalarlist *params);
+	virtual void setAnalyser(Analyser *a);
+};
+
 #endif /* !__HTPEIMG_H__ */
