@@ -2,7 +2,7 @@
  *	HT Editor
  *	relfile.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -31,17 +31,20 @@
 class ht_reloc_file: public ht_layer_streamfile {
 protected:
 	ht_tree *relocs;
+	bool enabled;
 
 /* new */
-	virtual void	reloc_apply(ht_data *reloc, byte *data);
-	virtual void	reloc_unapply(ht_data *reloc, byte *data);
+	virtual void	reloc_apply(ht_data *reloc, byte *data) = 0;
+	virtual bool	reloc_unapply(ht_data *reloc, byte *data) = 0;
 public:
 		   void	init(ht_streamfile *streamfile, bool own_streamfile);
 	virtual void	done();
 /* overwritten */
+		   void	finalize();
 		   void	insert_reloc(FILEOFS o, ht_data *reloc);
 	virtual UINT	read(void *buf, UINT size);
-	virtual UINT	write(void *buf, UINT size);
+	virtual int	vcntl(UINT cmd, va_list vargs);
+	virtual UINT	write(const void *buf, UINT size);
 };
 
 #endif /* __RELFILE_H__ */
