@@ -2,7 +2,7 @@
  *	HT Editor
  *	srt_x86.h
  *
- *	Copyright (C) 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 2001, 2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -23,6 +23,6 @@
 
 #include "srt.h"
 
-void srt_x86(analyser *analy, ADDR addr);
+void srt_x86(Analyser *analy, Address *addr);
 
 #endif /* __SRT_X86_H__ */
